/**
 * CS171 Basic Processing Example code.
 * User interaction - mouse and keyboard.
 * 
 * Written by Byron Hood.
 */
 
int HEIGHT = 400;
int WIDTH = 400;
int dotSize = 2;
int lineWidth = 2;
String currentText = "";

/**
 * Do the same setup as in the Shapes example, except load a new font this time.
 */
void setup( ) {
  size( WIDTH, HEIGHT );   // Set the size of the window.
  background( 196 );       // Set the background of the window.
  
  // Load 64-point Sans-Serif with smoothing and set it as the current text font.
  // All font loading [calls to createFont()/loadFont()] should be done in setup(),
  // because otherwise it'll slow your visualization down a lot.
  PFont myFont = createFont( "sans-serif", 64, true );
  textFont( myFont );
}

/**
 * Draw the same basics as the Shapes example, with dots on the background and a
 * large square in the middle.
 */
void draw( ) {
  background( 196 );
  strokeWeight( dotSize );
  stroke( 128, 32, 32 );
  for ( int x = 0; x < WIDTH; x += 10 ) {
    for ( int y = (x % 20) / 2; y < HEIGHT; y += 10 ) {
      point( x, y );
    }
  }
  
  fill( 255 ); // Set the shape fill color to white.
  strokeWeight( lineWidth );
  stroke( 32, 32, 128 );
  rectMode( CORNERS );
  rect( WIDTH / 4, HEIGHT / 4, 3 * WIDTH / 4, 3 * HEIGHT / 4 );
  
  fill( 0 );        // Set the text fill color to black.
  textAlign( CENTER, CENTER ); // Coordinates given for text will correspond to the
                               // center of the text, horizontally and vertically.
  textSize( 32 );   // Set the text size to 32px
  text( currentText, WIDTH / 2, HEIGHT / 2 );
}

/**
 * This is a special function: it will be called whenever a mouse button is clicked.
 */
void mouseClicked( ) {
  int m_x = mouseX; // mouseX is a special constant that has the mouse's x-coordinate.
  int m_y = mouseY; // mouseY has the mouse's y-coordinate.
  if ( mouseButton == LEFT ) {
    currentText = "( " + m_x + ", " + m_y + " )";
  } else if ( mouseButton == RIGHT ) {
    currentText = "";
  }
  redraw( ); // Force Processing to call draw() again.
}
 
 /**
  * This is also a special function. It will be called whenever a key is typed.
  */
void keyTyped( ) {
  char k = key; // Key contains the most recent key pressed.
  int m_x = mouseX;
  int m_y = mouseY;
  if ( k == 'd' ) {
    dotSize = max( 0, dotSize - 1 );
  } else if ( k == 'D' ) {
    dotSize = min( 20, dotSize + 1 );
  } else if ( k == 'l' ) {
    lineWidth = max( 0, lineWidth - 1 );
  } else if ( k == 'L' ) {
    lineWidth = min( 20, lineWidth + 1 );
  }
  currentText = "key: '" + k + "'\n( " + m_x + ", " + m_y + " )";
  redraw( ); // Force Processing to call draw() again.
}
